// =====================================================================
//  main.c
// =====================================================================

//  Author:         (c) 2013 Camil Demetrescu
//  License:        See the end of this file for license information
//  Created:        July 11, 2013
 
//  Last changed:   $Date: 2013/07/11 12:10:23 $
//  Changed by:     $Author: demetres $
//  Revision:       $Revision: 1.00 $


#include <stdio.h>
#include <stdlib.h>

#include "clut.h"
#include "pgm.h"
#include "resize.h"

#define IMG_DIR     "img"
#define HOST_DIR    "img_host"
#define DEVICE_DIR  "img_device"
#define PROG_NAME   "resize.cl"
#define MAX_LEN     1024


// ---------------------------------------------------------------------
// test_correctness
// ---------------------------------------------------------------------
// compare the two images

void test_correctness(unsigned char* img1, 
                      unsigned char* img2, 
                      int w, int h) {
    int x, y, err = 0;

    for (y=0; y<h; y++)
        for (x=0; x<w; x++)
            if (img1[y*w+x] != img2[y*w+x]) { err = 1; break; }

    printf("    Correctness test: %s\n", !err ? "PASSED" : "FAILED ###");
}


// ---------------------------------------------------------------------
// save_image
// ---------------------------------------------------------------------
// save image to disk

void save_image(unsigned char* img, int w, int h, 
                char* res_dir, char* name) {

    char img_path[MAX_LEN]; // pathname of output image file

    // build pathname of output image file
    sprintf(img_path, "%s/%s", res_dir, name);

    // save output image
    printf("Saving %dx%d image: %s\n", w, h, img_path);
    if (pgm_save(img, h, w, img_path))
        clut_panic("failed to save output image file");
}


// ---------------------------------------------------------------------
// run_test
// ---------------------------------------------------------------------
// resize input image
void run_test(char* infile, char* outfile, clut_device* dev) {

    double th, td;
    int w, h, err;
    unsigned char *in, *out_host, *out_device;
    char in_path[MAX_LEN]; // pathname of input image file

    // build pathname of input image file
    sprintf(in_path, "%s/%s", IMG_DIR, infile);

    // load image file in pgm format
    printf("Open image file: %s\n", in_path);
    err = pgm_load(&in, &h, &w, in_path);
    if (err) clut_panic("failed to load input image file");
    else     printf("Loaded %dx%d image\n", w, h);

    // allocate correct output image to be computed by host
    out_host = malloc((w/2)*(h/2)*sizeof(unsigned char));
    if (out_host == NULL) 
        clut_panic("failed to allocate host-computed output image on host");

    // allocate correct output image to be computed by device
    out_device = malloc((w/2)*(h/2)*sizeof(unsigned char));
    if (out_device == NULL) 
        clut_panic("failed to allocate device-computed output image on host");

    // resize image on host
    resize_host(in, out_host, w, h, &th);

    // resize image on device
    resize_device(in, out_device, w, h, dev, &td);

    // print running times
    printf("Resizing:\n");
    printf("    CPU took %.3f msec\n",    th*1E03);
    printf("    Device took %.3f msec\n", td*1E03);
    
    // perform correctness test
    test_correctness(out_host, out_device, w/2, h/2);

    // saving images to disk
    save_image(out_host,   w/2, h/2, HOST_DIR,   outfile);
    save_image(out_device, w/2, h/2, DEVICE_DIR, outfile);

    printf("\n");

    // cleanup
    free(in);
    free(out_host);
    free(out_device);
}


// ---------------------------------------------------------------------
// main
// ---------------------------------------------------------------------
int main(int argc, char** argv) {

    clut_device dev;                // device structure
    char*       infile;             // name of the input file
    char*       outfile;            // name of the output file

    // check number of input parameters
    if (argc < 2) {
        fprintf(stderr, "usage: ./%s infile outfile\n", argv[0]);
        exit(1);
    }
    
    // get input parameters
    infile  = argv[1];
    outfile = argv[2];

    // open device
    clut_open_device(&dev, PROG_NAME);

    // print device info
    clut_print_device_info(&dev);

    // run test
    run_test(infile, outfile, &dev);
    
    // cleanup
    clut_close_device(&dev);

    return 0;
}


// Copyright (C) 2013 Camil Demetrescu
  
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
  
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
  
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
// USA
